<?php

namespace Drupal\navigation_blocks;

use Drupal\Core\Link;
use Drupal\Core\Routing\CurrentRouteMatch;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Class BackButtonManager
 *
 * @package Drupal\navigation_blocks
 */
class BackButtonManager {

  use StringTranslationTrait;

  /**
   * The request stack to get the request object from.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * Block Path Matcher.
   *
   * @var \Drupal\navigation_blocks\PathMatcher
   */
  protected $blockPathMatcher;

  /**
   * CurrentRouteMatch service.
   *
   * @var \Drupal\Core\Routing\CurrentRouteMatch
   */
  protected $currentRouteMatch;

  /**
   * Constructs a new RefererBackButtonBlock object.
   *
   * @param \Drupal\Core\Routing\CurrentRouteMatch $currentRouteMatch
   *   Current Route Match.
   * @param \Symfony\Component\HttpFoundation\RequestStack $requestStack
   *   Request Stack.
   * @param \Drupal\navigation_blocks\PathMatcher $blockPathMatcher
   *   Block Path Matcher.
   */
  public function __construct(CurrentRouteMatch $currentRouteMatch, RequestStack $requestStack, PathMatcher $blockPathMatcher) {
    $this->requestStack = $requestStack;
    $this->blockPathMatcher = $blockPathMatcher;
    $this->currentRouteMatch = $currentRouteMatch;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('current_route_match'),
      $container->get('request_stack'),
      $container->get('navigation_blocks.path_matcher')
    );
  }

  /**
   * Get the preferred link.
   *
   * @param string $preferredPaths
   *   Preferred paths to navigate to.
   * @param string $backButtonText
   *   Text for the back link.
   * @param bool $useJavascript
   *   Use javascript for the back link.
   *
   * @return array
   *   Render array for the link.
   */
  public function getPreferredLink($preferredPaths, $backButtonText, $useJavascript = FALSE) {
    $refererPath = $this->getRefererPath();
    if (empty($refererPath)) {
      return [];
    }

    if (!empty($preferredPaths) && !$this->blockPathMatcher->matchPath($refererPath, $preferredPaths)) {
      return [];
    }

    $url = Url::fromUserInput($refererPath);
    if (!$this->blockPathMatcher->validateCurrentPath($url)) {
      return [];
    }

    return $this->getLink($url, $backButtonText, $useJavascript);
  }

  /**
   * Add link element attributes such as classes to the link.
   *
   * @param array $link
   *   Render array for the link.
   * @param bool $useJavascript
   *   Use javascript for the back link.
   *
   */
  public function addLinkAttributes(array &$link, $useJavascript = FALSE) {
    $link['#options']['attributes']['class'][] = 'back-button';

    if ($useJavascript) {
      $link['#options']['attributes']['class'][] = 'js-history-back';
      $link['#attached'] = [
        'library' => [
          'navigation_blocks/history-back',
        ],
      ];
    }
  }

  /**
   * Get the link.
   *
   * @param \Drupal\Core\Url $url
   *   Url to link to.
   * @param $text
   *   Text to show on the link.
   * @param $useJavascript
   *   Use javascript for the back link.
   *
   * @return array
   *   Render array for the link.
   */
  public function getLink(Url $url, $text, $useJavascript = FALSE) {
    if ($url->isRouted() && $this->currentRouteMatch->getRouteObject()
        ->getPath() === $url->getInternalPath()) {
      return [];
    }

    $link = Link::fromTextAndUrl($text, $url)->toRenderable();
    $this->addLinkAttributes($link, $useJavascript);
    return $link;
  }

  /**
   * Get the referer.
   *
   * @return string
   *   The referer.
   */
  public function getRefererPath() {
    $headers = $this->requestStack->getCurrentRequest()->headers->all();
    if (!isset($headers['referer'])) {
      return NULL;
    }
    return $this->stripBaseUrl($headers['referer'][0]);
  }

  /**
   * Strips the base URL from the referer.
   *
   * @param string $referer
   *   The referer.
   *
   * @return string
   *   The referer with the base URL stripped
   */
  public function stripBaseUrl($referer) {
    global $base_url;
    return str_replace($base_url, '', $referer);
  }

  /**
   * Check if the current path is a canonical path of an entity.
   *
   * @return bool
   *   TRUE if it is a canonical path.
   */
  public function validateCanonicalPath() {
    if (!strpos($this->currentRouteMatch->getRouteName(), '.canonical')) {
      return FALSE;
    }

    return TRUE;
  }
}