<?php

namespace Drupal\navigation_blocks;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityFieldManager;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeBundleInfo;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class EntityButtonManager
 *
 * @package Drupal\navigation_blocks
 */
class EntityButtonManager {

  use StringTranslationTrait;

  /**
   * EntityTypeManager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * EntityTypeBundleInfo.
   *
   * @var \Drupal\Core\Entity\EntityTypeBundleInfo
   */
  protected $entityTypeBundleInfo;

  /**
   * EntityFieldManager.
   *
   * @var \Drupal\Core\Entity\EntityFieldManager
   */
  protected $entityFieldManager;

  /**
   * Constructs a new RefererBackButtonBlock object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   Entity Type Manager.
   * @param \Drupal\Core\Entity\EntityTypeBundleInfo $entityTypeBundleInfo
   *   Entity Type Bundle Info.
   * @param \Drupal\Core\Entity\EntityFieldManager $entityFieldManager
   *   Entity Field Manager.
   */
  public function __construct(EntityTypeManagerInterface $entityTypeManager, EntityTypeBundleInfo $entityTypeBundleInfo, EntityFieldManager $entityFieldManager) {
    $this->entityTypeManager = $entityTypeManager;
    $this->entityTypeBundleInfo = $entityTypeBundleInfo;
    $this->entityFieldManager = $entityFieldManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('entity_type.bundle.info'),
      $container->get('entity_field.manager')
    );
  }

  /**
   * Get the entity reference field options.
   *
   * @param \Drupal\Core\Entity\EntityTypeInterface $entityType
   *   Entity type.
   *
   * @return string[]
   *   Entity reference field options.
   */
  public function getEntityReferenceFieldOptions(EntityTypeInterface $entityType) {
    $options = [];

    $bundles = array_keys($this->entityTypeBundleInfo->getBundleInfo($entityType->id()));
    foreach ($bundles as $bundle) {
      /** @var \Drupal\Core\Field\FieldDefinitionInterface[] $fields */
      $fields = $this->entityFieldManager->getFieldDefinitions($entityType->id(), $bundle);
      foreach ($fields as $field) {
        if ($field->getType() !== 'entity_reference') {
          continue;
        }
        $options[$field->getName()] = $field->getLabel();
      }
    }
    return $options;
  }

  /**
   * Get the reversed entity reference field options.
   *
   * @param \Drupal\Core\Entity\EntityTypeInterface $entityType
   *   Entity type.
   *
   * @return string[]
   *   Reversed entity reference field options.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function getReversedEntityReferenceFieldOptions(EntityTypeInterface $entityType) {
    /** @var \Drupal\field\FieldStorageConfigInterface[] $referenceFields */
    $referenceFields = $this->entityTypeManager->getStorage('field_storage_config')
      ->loadByProperties(
        [
          'settings' => [
            'target_type' => $entityType->id(),
          ],
          'type' => 'entity_reference',
          'deleted' => FALSE,
          'status' => 1,
        ]
      );

    $options = [];
    foreach ($referenceFields as $referenceField) {
      $fieldEntityType = $this->getEntityType($referenceField->getTargetEntityTypeId());
      $fieldName = $referenceField->getName();


      /** @var \Drupal\Core\Field\FieldConfigInterface[] $fieldInstances */
      $fieldInstances = $this->entityTypeManager->getStorage('field_config')
        ->loadByProperties(
          [
            'field_name' => $referenceField->getName(),
            'entity_type' => $fieldEntityType->id(),
          ]
        );

      foreach ($fieldInstances as $fieldInstance) {
        $bundle = $fieldInstance->getTargetBundle();
        $options[$fieldEntityType->id() . ':' . $bundle . ':' . $fieldName] = $fieldEntityType->getLabel() . ' (' . $bundle . ') : ' . $fieldInstance->label();
      }
    }

    return $options;
  }

  /**
   * Get reversed entity reference entity.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   Content Entity.
   * @param string $entityTypeId
   *   Entity type id.
   * @param string $bundle
   *   Entity type bundle.
   * @param string $fieldName
   *   Entity reference field name.
   *
   * @return \Drupal\Core\Entity\ContentEntityInterface
   *   Referenced content entity.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function getReversedEntityReferenceEntity(EntityInterface $entity, $reversedEntityTypeId, $reversedBundle, $reversedFieldName) {
    $reversedEntityType = $this->getEntityType($reversedEntityTypeId);
    $entityType = $entity->getEntityType();
    $properties = [
      $reversedFieldName . '.entity.' . $entityType->getKey('id') => $entity->id(),
    ];

    if (!empty($reversedBundle)) {
      $properties[$reversedEntityType->getKey('bundle')] = $reversedBundle;
    }

    $entities = $this->entityTypeManager->getStorage($reversedEntityTypeId)
      ->loadByProperties($properties);
    return reset($entities);
  }

  /**
   * Get the referenced entity.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   Content Entity.
   * @param string $fieldName
   *   Name of the entity reference field name.
   *
   * @return \Drupal\Core\Entity\ContentEntityInterface
   *   Referenced Entity.
   */
  public function getReferencedEntity(ContentEntityInterface $entity, $fieldName) {
    /** @var \Drupal\Core\Field\EntityReferenceFieldItemList $entityReferenceFieldItemList */
    $entityReferenceFieldItemList = $entities = $entity->get($fieldName);
    /** @var \Drupal\Core\Entity\ContentEntityInterface[] $entities */
    $entities = $entityReferenceFieldItemList->referencedEntities();
    return reset($entities);
  }

  /**
   * Get the entity type definition.
   *
   * @param string $entityTypeId
   *   Entity type id.
   *
   * @return \Drupal\Core\Entity\EntityTypeInterface
   *   Entity Type definition.
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function getEntityType($entityTypeId) {
    return $this->entityTypeManager->getDefinition($entityTypeId);
  }
}