<?php

namespace Drupal\navigation_blocks\Plugin\Block;

use Drupal\Core\Form\FormStateInterface;
use Drupal\navigation_blocks\BackButtonManager;
use Drupal\navigation_blocks\EntityButtonManager;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a 'EntityReferenceBackButton' block.
 *
 * @Block(
 *  id = "entity_reference_back_button",
 *  deriver = "Drupal\navigation_blocks\Plugin\Deriver\EntityReferenceBackButtonDeriver"
 * )
 */
class EntityReferenceBackButton extends BackButton {

  /**
   * @var \Drupal\navigation_blocks\EntityButtonManager
   */
  protected $entityButtonManager;

  /**
   * Constructs a new RefererBackButtonBlock object.
   *
   * @param array $configuration
   * @param $pluginId
   * @param $pluginDefinition
   * @param \Drupal\navigation_blocks\BackButtonManager $backButtonManager
   *   Back Button Manager.
   * @param \Drupal\navigation_blocks\EntityButtonManager $entityButtonManager
   *   Entity Button Manager.
   */
  public function __construct(array $configuration, $pluginId, $pluginDefinition, BackButtonManager $backButtonManager, EntityButtonManager $entityButtonManager) {
    parent::__construct($configuration, $pluginId, $pluginDefinition, $backButtonManager);
    $this->entityButtonManager = $entityButtonManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $pluginId, $pluginDefinition) {
    return new static(
      $configuration,
      $pluginId,
      $pluginDefinition,
      $container->get('navigation_blocks.back_button_manager'),
      $container->get('navigation_blocks.entity_button_manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    $configuration = parent::defaultConfiguration();
    $configuration['entity_reference_field'] = '';
    return $configuration;
  }

  /**
   * {@inheritdoc}
   * @throws \Drupal\Core\Entity\EntityMalformedException
   */
  public function build() {
    $build = $this->backButtonManager->getPreferredLink($this->getPreferredBackPaths(), self::BACK_BUTTON_TEXT, $this->useJavascript());

    if (!empty($build)) {
      return $build;
    }

    /** @var \Drupal\Core\Entity\EntityInterface $entity */
    $entity = $this->getContextValue('entity');

    $configuration = $this->getConfiguration();
    $entityReferenceFieldName = $configuration['entity_reference_field'];

    $referencedEntity = $this->entityButtonManager->getReferencedEntity($entity, $entityReferenceFieldName);
    if ($referencedEntity) {
      $link = $referencedEntity->toLink()->toRenderable();
      $this->backButtonManager->addLinkAttributes($link);
      return $link;
    }

    return $this->backButtonManager->getLink($this->getLinkUrl(), $this->getLinkText());
  }

  /**
   * {@inheritdoc}
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   */
  public function blockForm($form, FormStateInterface $formState) {
    $form = parent::blockForm($form, $formState);

    /** @var \Drupal\Core\Plugin\Context\EntityContextDefinition $entityContextDefinition */
    $entityContextDefinition = $this->getContextDefinition('entity')->getDataDefinition();
    /** @var string $entityTypeId */
    $entityTypeId = $entityContextDefinition->getConstraint('EntityType');
    $entityType = $this->entityButtonManager->getEntityType($entityTypeId);

    $entityFieldReferenceOptions = $this->entityButtonManager->getEntityReferenceFieldOptions($entityType);

    $form['entity_reference_field'] = [
      '#type' => 'select',
      '#empty_value' => '',
      '#title' => t('Entity Reference Field'),
      '#description' => t(''),
      '#default_value' => $this->configuration['entity_reference_field'],
      '#options' => $entityFieldReferenceOptions,
      '#required' => TRUE,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    parent::blockSubmit($form, $form_state);
    $this->setConfigurationValue('entity_reference_field', $form_state->getValue('entity_reference_field'));
  }
}
